#include "frameBuffer.h"

#include "FL/gl.h"
#include <iostream.h>

#include "globals.h"
#include <math.h>
#include <float.h>

#include <tiffio.h>
#include <fstream.h>


FrameBuffer::FrameBuffer(int u0, int v0, int _w, int _h) : 
  w(_w), h(_h), Fl_Gl_Window(u0, v0, _w, _h, 0) {

  pix = new unsigned int[w*h];

}

void FrameBuffer::draw() {

  if (pix)
    glDrawPixels(w, h, GL_RGBA, GL_UNSIGNED_BYTE, pix);
  else
    cerr << "attempted to post bad pixels in FrameBuffer::PostSWFB()" << endl;

}

void FrameBuffer::show() {

  if (pix)
    size(w, h);
  Fl_Gl_Window::show();
  
}

int FrameBuffer::handle(int event) {

  switch (event) {
  case FL_RELEASE: {
    int mouseu = Fl::event_x();
    int mousev = Fl::event_y();
    cerr << "INFO: released mouse button at: " << 
      mouseu << " " << mousev << endl;
    break;
  }
  case FL_MOVE: {
    int mouseu = Fl::event_x();
    int mousev = Fl::event_y();
    scene->MouseMove(mouseu, mousev);
    break;
  }
  case FL_KEYBOARD: {
    int key = Fl::event_key();
    scene->KeyPress(key);
    break;
  }
  default: {
    // events I don't handle
  }
  }

  return 1;

}

void FrameBuffer::Set(int u, int v, unsigned int color) {

  pix[(h-1-v)*w+u] = color;

}

void FrameBuffer::SetGuarded(int u, int v, unsigned int color) {

  if (u > 0 && u < w && v > 0 && v < h) {
    Set(u, v, color);
  }

}

void FrameBuffer::Clear(unsigned int clearColor) {

  for (int v = 0; v < h; v++) {
    for (int u = 0; u < w; u++) {
      Set(u, v, clearColor);
    }
  }

}

void FrameBuffer::MakeChecker(int checkerWidth, unsigned int *colors) {

  for (int v = 0; v < h; v++) {
    for (int u = 0; u < w; u++) {
      unsigned int currColor = colors[0];
      int cu = u/checkerWidth;
      int cv = v/checkerWidth;
      if ((cu+cv)%2) {
        currColor = colors[1];
      }
      Set(u, v, currColor);
    }
  }


}

void FrameBuffer::Setf(float uf, float vf, unsigned color) {

  int u = (int)uf;
  int v = (int)vf;
  SetGuarded(u, v, color);

}

void FrameBuffer::DrawPointDiagonalCross(float uf, float vf, 
  int size, unsigned int color) {


  int u = (int)uf;
  int v = (int)vf;
  for (int i = 0; i < size; i++) {
    SetGuarded(u-size/2+i, v-size/2+i, color);
    SetGuarded(u+size/2-i, v-size/2+i, color);
  }

}

void FrameBuffer::DrawPointSquare(float uf, float vf, 
  int size, unsigned int color) {


  int u = (int)uf;
  int v = (int)vf;
  for (int cv = v-size/2; cv <= v+size/2; cv++) {
    for (int cu = u-size/2; cu <= u+size/2; cu++) {
      SetGuarded(cu, cv, color);
    }
  }
}

void FrameBuffer::DrawSegment(float u0f, float v0f, float u1f, float v1f,
  unsigned int color) {

  float dv = v1f-v0f;
  float du = u1f-u0f;
  int nSteps;
  if (fabsf(dv) > fabsf(du)) {
    nSteps = (int) ceilf(fabsf(dv));
  }
  else {
    nSteps = (int) ceilf(fabsf(du));
  }

  float curruf = u0f;
  float currvf = v0f;
  for (int i = 0; i <= nSteps; i++) {
    Setf(curruf, currvf, color);
    curruf += du/(float)nSteps;
    currvf += dv/(float)nSteps;
  }

}

void FrameBuffer::DrawPointDiagonalCross3D(V3& point, PHC &phc, 
  int size, unsigned int color) {

  V3 projP;
  if (!phc.Project(point, projP))
    return;

  DrawPointDiagonalCross(projP[0], projP[1], size, color);

}

void FrameBuffer::DrawPointSquare3D(V3& point, PHC &phc, 
  int size, unsigned int color) {

  V3 projP;
  if (!phc.Project(point, projP))
    return;

  DrawPointSquare(projP[0], projP[1], size, color);

}


void FrameBuffer::DrawSegment3D(V3 point0, V3 point1, PHC &phc,
  unsigned int color) {


  V3 projP[2];
  if (!phc.Project(point0, projP[0]))
    return;
  if (!phc.Project(point1, projP[1]))
    return;

  DrawSegment(projP[0][0], projP[0][1], projP[1][0], projP[1][1],
    color);

}
